Skip to contents

A short display of the call and the augmented-regression coefficients; see summary.copreg for the full table with standard errors, endogeneity tests, and diagnostics.

Usage

# S3 method for class 'copreg'
print(x, digits = max(3L, getOption("digits") - 3L), ...)

Arguments

x

a fitted model of class "copreg".

digits

number of significant digits to print.

...

currently unused.

Value

x, invisibly. Called for the side effect of printing.

References

Qian, Y., A. Koschmann, and H. Xie (2025). A practical guide to endogeneity correction using copulas. Journal of Marketing.

Examples

set.seed(1)
n  <- 150
w  <- rnorm(n)
p  <- 0.4 * w + rt(n, df = 3)
xi <- 0.5 * p + rnorm(n)
y  <- 1 + 2 * p + 1.5 * w + xi
dat <- data.frame(y = y, p = p, w = w)

fit <- endogCopula:::.copreg_fit(
  formula = y ~ p | w, data = dat,
  ctor = endogCopula:::.ctor_twostage(TRUE),
  method = "2sCOPE", cdf = "rank.n", ties = "max",
  nboots = 25, verbose = FALSE)

print(fit)
#> 
#> Copula endogeneity correction: 2sCOPE 
#> 
#> Coefficients:
#> (Intercept)            p            w        p_cop  
#>      0.8776       2.8387       1.4293      -0.5092  
#>